#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using System.Collections.Generic;
using HeuristicLab.Problems.DataAnalysis.Symbolic;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  public abstract class FixedDataAnalysisAlgorithm<T> : Algorithm,
    IDataAnalysisAlgorithm<T>,
    IStorableContent
    where T : class, IDataAnalysisProblem {
    public string Filename { get; set; }

    #region Properties
    public override Type ProblemType {
      get { return typeof(T); }
    }
    public new T Problem {
      get { return (T)base.Problem; }
      set { base.Problem = value; }
    }
    [Storable]
    private ResultCollection results;
    public override ResultCollection Results {
      get { return results; }
    }
    #endregion

    [StorableConstructor]
    protected FixedDataAnalysisAlgorithm(bool deserializing) : base(deserializing) { }
    protected FixedDataAnalysisAlgorithm(FixedDataAnalysisAlgorithm<T> original, Cloner cloner)
      : base(original, cloner) {
      results = cloner.Clone(original.Results);
    }
    public FixedDataAnalysisAlgorithm()
      : base() {
      results = new ResultCollection();
    }

    public override void Prepare() {
      if (Problem != null) base.Prepare();
      results.Clear();
      OnPrepared();
    }

    public override void Start() {
      base.Start();
      OnStarted();
      Run();
      OnStopped();
    }

    #region Events
    protected override void OnProblemChanged() {
      Problem.Reset += new EventHandler(Problem_Reset);
      base.OnProblemChanged();
    }

    #endregion

    protected abstract void Run();
  }
}
